// ==========================================================================
// NumLock.h: interface for the CNumLock class
//
// Copyright:   1999 - Wilfried Roemer, Berlin, all rights reserved
//
// An object of this class can be used in an application to ensure that the
// NumLock key retains a given state (On or Off).
//
// Usage:
//
// - add a single CNumLock object to the application as a private member:
//
//      CNumLock m_NumLock;
//
// - initialize the CNumLock object in the application's constructor if
//   the default value (NumLock always On) is not applicable
//
// - override OnIdle and call CheckNumLockState() when the idle count
//   (OnIdle parameter) is 0:
//
//    	if( lCount <= 0 )
//        m_NumLock.CheckNumLockState();
//     	return CWinApp::OnIdle(lCount);
//
// - override PreTranslateMessage and call FilterMessages():
//
//      if( m_NumLock.FilterMessage( pMsg ))
//        return TRUE;
//      else
//        return CWinApp::PreTranslateMessage(pMsg);
//
// Notes:
//
// According to MSDN information, the method to toggle the NumLock key is
// different between Windows 9x and NT. The OS type is computed within the
// constructor as static information.
//
// The actual NumLock state is handled within the BIOS. The state changes
// when the NumLock key is depressed. In order to prohibit any key input
// while the NumLock key is in an undesired state, FilterMessage must be
// called in the application's PreTranslateMessage handler. That will
// test whether the depressed key is the NumLock key. If the NumLock key
// is down, all other key input will be ignored until the NumLock key goes
// up again. Then the state of the NumLock key will be checked. If necessary,
// another NumLock key down/up sequences will be injected which will then
// reset the state to the state required for that window.
//
// If the state of the NumLock key is changed while that window is not
// active, it appears that GetKeyState returns a valid value after the
// application has entered the idle loop. Therefore, OnIdle must be over-
// ridden and CheckNumLockState() must be called.
//
// The same method should work for the CapsLock and ScrollLock keys.
// ==========================================================================

#pragma once

class CNumLock  
  {
  private:

	OSVERSIONINFO m_OSVersionInfo; // needed to distinguish Windows 9x/NT

  int  m_iReqdNumLockState; // state of NumLock key required for this app
  bool m_bIgnoreKeyInput;   // true while NumLock key is down

  int  GetCurrNumLockState()const{ return GetKeyState( VK_NUMLOCK ) & 0x0001; };
	void ToggleNumLock();

  public:

  void CheckNumLockState();               // include in OnIdle
  bool FilterMessage( const MSG* pMsg );  // include in PreTranslateMessage

  CNumLock( bool bAlwaysOn = true );
	virtual ~CNumLock();
  };
